#
# Copyright (c) 2014, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

import os
import signal
import sys
import six
import subprocess
import logging
import re
from optparse import OptionParser
from javapackages.common.exception import JavaPackagesToolsException
from javapackages.common.config import get_buildroot


def kill_parent_process(rpmconf):
    if not rpmconf:
        return
    # mock may kill us immediately after rpmbuild dies, before output
    # is flushed. To avoid this race condiditon we must explicitly
    # flush any pending output before trying to kill parent.
    sys.stdout.flush()
    sys.stderr.flush()
    # rpmbuild ignores non-zero exit codes, but this is bad. Make sure
    # the build fails and doesn't silently ignore problems
    try:
        os.kill(rpmconf.rpm_pid, signal.SIGTERM)
    except OSError:
        pass
    sys.exit(1)


def args_to_unicode(args):
    if six.PY2:
        for index, arg in enumerate(args):
            args[index] = arg.decode(sys.getfilesystemencoding())
    return args


def execute_command(command, input=None):
    proc = subprocess.Popen([command], shell=True,
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                            stdin=subprocess.PIPE, universal_newlines=True)
    stdout, stderr = proc.communicate(input=input)
    proc.wait()
    return proc.returncode, stdout, stderr


def command_exists(cmd):
    command = "type {cmd}".format(cmd=cmd)
    rc, _, _ = execute_command(command)
    return True if not rc else False


def init_rpmgen(argv):
    return _parse_rpmgen_args(argv)


def _parse_argv(argv):
    parser = OptionParser()
    parser.add_option("--cachedir", dest="cachedir")
    parser.add_option("--rpm-pid", dest="rpm_pid")
    parser.add_option("--scl", dest="scl", default=None)

    return parser.parse_args()


def _parse_rpmgen_args(argv):
    options = _parse_argv(argv)[0]

    if not options.cachedir:
        raise JavaPackagesToolsException("Missing option: --cachedir")
    options.cachedir = _get_cachedir(options.cachedir)

    if not options.rpm_pid:
        raise JavaPackagesToolsException("Missing option: --rpm-pid")
    options.rpm_pid = int(options.rpm_pid)

    return options


def _get_cachedir(path, create_if_not_exists=True):
    cachedir_path = os.path.join(path, ".javapackages_cache")
    if not os.path.exists(cachedir_path) and create_if_not_exists:
        os.makedirs(cachedir_path)
    return cachedir_path


def get_logger(name):
    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)
    handler = logging.StreamHandler()
    formatter = logging.Formatter("[%(levelname)s %(name)s] %(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    return logger


def get_buildroot_files(regexp=None,
                        inside_buildroot_only=True,
                        exclude_broken_symlinks=True,
                        buildroot_path=None):
    if not buildroot_path:
        buildroot_path = get_buildroot()
    files = set()
    for dirpath, dirnames, filenames in os.walk(buildroot_path):
        for filename in filenames:
            f = os.path.realpath(os.path.join(dirpath, filename))

            if inside_buildroot_only:
                if not f.startswith(buildroot_path):
                    continue

            if exclude_broken_symlinks:
                if not os.path.exists(f):
                    continue

            if regexp:
                match = re.search(regexp, f)
                if not match:
                    continue
            files.add(f)
    return files



#
# Copyright (c) 2015, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Stanislav Ochotnicky <sochotnicky@redhat.com>
#           Michal Srb <msrb@redhat.com>

import re
import six

import javapackages.maven.pomreader as POMReader
import javapackages.common.strutils as Printer
from javapackages.common.exception import JavaPackagesToolsException
from lxml.etree import Element, SubElement, tostring


class ArtifactException(JavaPackagesToolsException):
    pass


class ArtifactFormatException(JavaPackagesToolsException):
    pass


class ArtifactValidationException(JavaPackagesToolsException):
    pass


class AbstractArtifact(object):

    def get_mvn_str(self):
        m = self.__get_members()
        return Printer.get_mvn_str(m['groupId'],
                                   m['artifactId'],
                                   ext=m['extension'],
                                   cla=m['classifier'],
                                   ver=m['version'])

    def get_rpm_str(self, namespace=None, compat_ver=None, pkg_ver=None):
        m = self.__get_members()
        return Printer.get_rpm_str(m['groupId'],
                                   m['artifactId'],
                                   ext=m['extension'],
                                   cla=m['classifier'],
                                   namespace=namespace,
                                   compat_ver=compat_ver,
                                   pkg_ver=pkg_ver)

    @staticmethod
    def get_parts_from_mvn_str(mvnstr):
        tup = mvnstr.split(":")

        # groupId and artifactId are always present
        if len(tup) < 2:
            raise ArtifactFormatException("Artifact string '{mvnstr}' does not "
                                          "contain ':' character. Can not parse"
                                          .format(mvnstr=mvnstr))

        if len(tup) > 5:
            raise ArtifactFormatException("Artifact string '{mvnstr}' contains "
                                          "too many colons. Can not parse"
                                          .format(mvnstr=mvnstr))

        parts = {'groupId': '',
                 'artifactId': '',
                 'extension': '',
                 'classifier': '',
                 'version': ''}

        parts['groupId'] = tup[0]
        parts['artifactId'] = tup[1]
        parts['extension'] = tup[2] if len(tup) >= 4 else ""
        parts['classifier'] = tup[3] if len(tup) >= 5 else ""
        parts['version'] = tup[-1] if len(tup) >= 3 else ""

        return parts

    def get_xml_element(self, root_element_name=None):
        """
        Return XML Element node representation of the Artifact
        """
        if root_element_name is None:
            clsname = self.__class__.__name__
            root_element_name = clsname[0].lower() + clsname[1:]
        root = Element(root_element_name)

        members = self.__get_members()

        for key in members:
            if (key == "extension" and members[key] == "jar" and
               hasattr(self, "_default_extension") and
               getattr(self, "_default_extension") is True):
                continue
            if members[key]:
                item = SubElement(root, key)
                item.text = members[key]
        return root

    def get_xml_str(self, root_element_name=None):
        """
        Return XML formatted string representation of the Artifact
        """
        if root_element_name is None:
            clsname = self.__class__.__name__
            root_element_name = clsname[0].lower() + clsname[1:]

        root = self.get_xml_element(root_element_name)
        return tostring(root, pretty_print=True)

    def get_artifact(self, extension="", classifier="", version=""):
        m = self.__get_members()

        return Artifact(m['groupId'], m['artifactId'],
                        extension or m['extension'],
                        classifier or m['classifier'],
                        version or m['version'])

    def merge_with(self, artifact):
        if type(self) != type(artifact):
            raise Exception("Unable to merge artifacts of different types. "
                            "{this} cannot be merged with {other}"
                            .format(this=type(self), other=type(artifact)))

        if self.compare_to(artifact):
            for member in self.__dict__:
                if not member.startswith('_'):
                    # for "scope":
                    # copy value from parent artifact only if this object
                    # contains default/implicit value
                    if (member == "scope" and
                       hasattr(self, "_default_" + member) and
                       getattr(self, "_default_" + member)):
                        setattr(self, member, getattr(artifact, member))
                        # if the copied value was not a default value in parent,
                        # then we have a definitive value and we should never
                        # override it again
                        if not getattr(artifact, "_default_" + member):
                            setattr(self, "_default_" + member, False)
                    # for fields other than "scope":
                    # copy value from parent artifact, if current value is empty
                    elif not getattr(self, member):
                        setattr(self, member, getattr(artifact, member))

    def update_from(self, artifact):
        if self.compare_to(artifact):
            for member in self.__dict__:
                if not member.startswith('_') and not getattr(self, member):
                    setattr(self, member, getattr(artifact, member))
            return True
        return False

    def compare_to(self, artifact):
        if type(self) != type(artifact):
            return False
        this = self.__get_significant_members(ignore_version=True)
        other = artifact.__get_significant_members(ignore_version=True)
        if this == other:
            return True
        return False

    def interpolate(self, properties):
        leftovers = []
        for member in self.__dict__:
            if (not member.startswith('_') and
               getattr(self, member) and
               isinstance(getattr(self, member), six.string_types)):
                    curr_value = getattr(self, member)
                    prog = re.compile("\$\{([^}]+)\}")
                    props = prog.findall(curr_value)
                    for key in props:
                        try:
                            prop_value = properties[key]
                            curr_value = curr_value.replace("${{{prop}}}"
                                                            .format(prop=key), prop_value)
                            setattr(self, member, curr_value)
                        except KeyError:
                            leftovers.append(key)
        return leftovers

    def __unicode__(self):
        return six.text_type(self.get_mvn_str())

    def __str__(self):
        return self.__unicode__()

    def __eq__(self, other):
        if type(other) is type(self):
            return self.__get_significant_members() == other.__get_significant_members()
        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        result = 0
        for key in self.__get_significant_members():
            var = getattr(self, key)
            result += var.__hash__()
        return result

    def __get_significant_members(self, ignore_version=False):
        m = {}
        for mname in ["groupId", "artifactId", "extension", "classifier"]:
            if hasattr(self, mname):
                m[mname] = getattr(self, mname)
            if not ignore_version and hasattr(self, "version"):
                m["version"] = getattr(self, "version")
        return m

    def __get_members(self):
        m = {'groupId': '',
             'artifactId': '',
             'extension': '',
             'classifier': '',
             'version': ''}

        for key in m:
            if hasattr(self, key):
                m[key] = getattr(self, key)

        return m


class Artifact(AbstractArtifact):
    """
    Simplified representation of Maven artifact for purpose of packaging

    Package consists mostly of simple properties and string formatting and
    loading functions to prevent code duplication elsewhere
    """

    def __init__(self, groupId, artifactId, extension="",
                 classifier="", version=""):
        self.groupId = groupId.strip()
        self.artifactId = artifactId.strip()
        self.extension = extension.strip() or "jar"
        self.classifier = classifier.strip()
        self.version = version.strip()

        self._default_extension = True
        if extension:
            self._default_extension = False

    @classmethod
    def merge_artifacts(cls, dominant, recessive):
        """
        Merge two artifacts into one. Information missing in dominant artifact
        will be copied from recessive artifact. Returns new merged artifact.
        """
        ret = cls(dominant.groupId, dominant.artifactId, dominant.extension,
                  dominant.classifier, dominant.version)
        for key in ("artifactId", "groupId", "extension", "version",
                    "classifier"):
            if not getattr(ret, key):
                setattr(ret, key, getattr(recessive, key))
        return ret

    def validate(self, allow_empty=True, allow_wildcards=True,
                 allow_backref=True):
        """
        Function to validate current state of artifact with regards to
        wildcards, empty parts and backreference usage
        """
        all_empty = True
        wildcard_used = False
        backref_used = False
        backref_re = re.compile(r'@\d+')
        for key in ("artifactId", "groupId", "extension", "version",
                    "classifier"):
            val = getattr(self, key)
            if not val:
                continue
            if val and key == "extension" and self._default_extension:
                continue
            else:
                all_empty = False
            if val.find('*') != -1:
                wildcard_used = True
            if backref_re.match(val):
                backref_used = True

        if not allow_empty and all_empty:
            raise ArtifactValidationException("All parts of artifact are empty")
        if not allow_wildcards and wildcard_used:
            raise ArtifactValidationException("Wildcard used in artifact")
        if not allow_backref and backref_used:
            raise ArtifactValidationException("Backreference used in artifact")
        return True

    @classmethod
    def from_xml_element(cls, xmlnode):
        """
        Create Artifact from xml.etree.ElementTree.Element as contained
        within pom.xml or a dependency map.
        """

        parts = {'groupId': '',
                 'artifactId': '',
                 'extension': '',
                 'classifier': '',
                 'version': ''}

        parts = POMReader.find_parts(xmlnode, parts)

        if not parts['groupId'] or not parts['artifactId']:
            raise ArtifactFormatException(
                "Empty groupId or artifactId encountered. "
                "This is a bug, please report it!")

        return cls(parts['groupId'], parts['artifactId'], parts['extension'],
                   parts['classifier'], parts['version'])

    @classmethod
    def from_mvn_str(cls, mvnstr):
        """
        Create Artifact from Maven-style definition

        The string should be in the format of:
           groupId:artifactId[:extension[:classifier]][:version]

        Where last part is always considered to be version unless empty
        """
        p = Artifact.get_parts_from_mvn_str(mvnstr)
        return cls(p['groupId'], p['artifactId'], p['extension'],
                   p['classifier'], p['version'])



#
# Copyright (c) 2014, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

import javapackages.common.config as config
from javapackages.metadata.metadata import Metadata, MetadataLoadingException
from javapackages.cache.cache import Cache

from copy import deepcopy

class MetadataCache(Cache):
    def __init__(self, rpmconf):
        super(MetadataCache, self).__init__(rpmconf)
        self._config_name = config.metadata_cache_f
        self._cache = self._read_cache()

        if self._cache is None:
            self._cache = self._process_buildroot()
            self._write_cache(self._cache)

    def _process_buildroot(self):
        # "path: Metadata" mapping
        cache = {}

        metadata_paths = self._find_paths()
        for path in metadata_paths:
            try:
                metadata = Metadata.create_from_file(path)
                if metadata:
                    cache.update({path: metadata})
            except MetadataLoadingException:
                continue

        return cache

    def _check_path(self, path):
        # TODO
        if "/usr/share/maven-metadata/" in path and path.endswith(".xml"):
            return True
        return False

    def get_artifact_for_path(self, path, can_be_dir=False):
        for metadata in self._cache.values():
            artifact = metadata.get_artifact_for_path(path,
                                                      can_be_dir=can_be_dir)
            if artifact:
                return artifact
        return None

    def get_metadata_for_path(self, path):
        try:
            return self._cache[path].copy()
        except KeyError:
            pass
        return None

    def get_provided_artifacts(self):
        artifacts = []
        for metadata in self._cache.values():
            artifacts.extend(deepcopy(metadata.artifacts))
        return artifacts

    def get_skipped_artifacts(self):
        artifacts = []
        for metadata in self._cache.values():
            artifacts.extend(deepcopy(metadata.skippedArtifacts))
        return artifacts

    def get_provided_osgi(self):
        bundles = []
        for metadata in self._cache.values():
            bundles += metadata.get_osgi_provides()
        return bundles



import unittest

from test_common import *


class TestMavenProv(unittest.TestCase):

    @mavenprov(["simple/buildroot/usr/share/maven-metadata/simple.xml"])
    def test_simple(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 2)
        assertIn(self, "ns-mvn(org.codehaus.plexus:plexus-ant-factory:1.0) = 1.0", sout)
        assertIn(self, "ns-mvn(org.codehaus.plexus:plexus-ant-factory:pom:1.0) = 1.0", sout)

    @mavenprov(["invalid/buildroot/usr/share/maven-metadata/invalid.xml"])
    def test_invalid(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)

    @mavenprov(["notxml/buildroot/usr/share/maven-metadata/not_xml.xml"])
    def test_not_xml(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)

    @mavenprov(["nonexistent_file_blablabla"])
    def test_nonexistent(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)

    @mavenprov(["single_ns/buildroot/usr/share/maven-metadata/single_ns.xml"])
    def test_single_ns(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 4)
        assertIn(self, "ns-mvn(org.codehaus.plexus:plexus-ant-factory:1.0) = 1.0", sout)
        assertIn(self, "ns-mvn(org.codehaus.plexus:plexus-ant-factory:pom:1.0) = 1.0", sout)
        assertIn(self, "ns-mvn(org.mortbay.jetty:jsp-2.1-glassfish:1.0) = 9.1.1.b60.25.p2", sout)
        assertIn(self, "ns-mvn(org.mortbay.jetty:jsp-2.1-glassfish:pom:1.0) = 9.1.1.b60.25.p2", sout)

    @mavenprov(["multi_ns/buildroot/usr/share/maven-metadata/multi_ns.xml"])
    def test_multi_ns(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 4)
        assertIn(self, "ns-mvn(org.codehaus.plexus:plexus-ant-factory:1.0) = 1.0", sout)
        assertIn(self, "ns-mvn(org.codehaus.plexus:plexus-ant-factory:pom:1.0) = 1.0", sout)
        assertIn(self, "ns2-mvn(org.mortbay.jetty:jsp-2.1-glassfish:1.0) = 9.1.1.b60.25.p2", sout)
        assertIn(self, "ns2-mvn(org.mortbay.jetty:jsp-2.1-glassfish:pom:1.0) = 9.1.1.b60.25.p2", sout)

    #@mavenprov(["no_version.xml"])
    #def test_no_version(self, stdout, stderr, return_value):
    #    self.assertNotEqual(return_value, 0)

    #@mavenprov(["simple.xml", "simple2.xml"])
    #def test_more_files(self, stdout, stderr, return_value):
    #    self.assertEqual(return_value, 0, stderr)
    #    sout = [x for x in stdout.split('\n') if x]
    #    self.assertEqual(len(sout), 4)
    #    assertIn(self, "ns-mvn(org.codehaus.plexus:plexus-ant-factory:1.0) = 1.0", sout)
    #    assertIn(self, "ns-mvn(org.codehaus.plexus:plexus-ant-factory:pom:1.0) = 1.0", sout)
    #    assertIn(self, "ns-mvn(org.mortbay.jetty:jsp-2.1-glassfish:6.0.18) = 9.1.1.b60.25.p2", sout)
    #    assertIn(self, "ns-mvn(org.mortbay.jetty:jsp-2.1-glassfish:pom:6.0.18) = 9.1.1.b60.25.p2", sout)

    #@mavenprov(["two_in_one.xml"])
    #def test_two_in_one(self, stdout, stderr, return_value):
    #    self.assertNotEqual(return_value, 0)

    @mavenprov(["non_compat/buildroot/usr/share/maven-metadata/non_compat.xml"])
    def test_non_compat(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 2)
        assertIn(self, "ns-mvn(org.codehaus.plexus:plexus-ant-factory) = 1.0", sout)
        assertIn(self, "ns-mvn(org.codehaus.plexus:plexus-ant-factory:pom:) = 1.0", sout)

    @mavenprov(["extension1/buildroot/usr/share/maven-metadata/extension1.xml"])
    def test_extension1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 1)
        assertIn(self, "ns-mvn(org.mortbay.jetty:jsp-2.1-glassfish:war:6.0.18) = 9.1.1.B60.25.p2", sout)

    @mavenprov(["extension2/buildroot/usr/share/maven-metadata/extension2.xml"])
    def test_extension2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 1)
        assertIn(self, "ns-mvn(org.mortbay.jetty:jsp-2.1-glassfish:6.0.18) = 9.1.1.B60.25.p2", sout)

    @mavenprov(["pom_extension/buildroot/usr/share/maven-metadata/pom_extension.xml"])
    def test_pom_extension(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 1)
        assertIn(self, "ns-mvn(org.mortbay.jetty:jsp-2.1-glassfish:pom:) = 6.0.18", sout)

    @mavenprov(["pom_namespace/buildroot/usr/share/maven-metadata/pom_namespace.xml"])
    def test_namespace_rhbz1017271(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 1)
        assertIn(self, "maven31-mvn(org.apache.maven:apache-maven:pom:) = 3.1.1", sout)

    @mavenprov(["pom_compat/buildroot/usr/share/maven-metadata/pom_compat.xml"])
    def test_compat_version_in_artifact(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 1)
        assertIn(self, "mvn(org.apache.maven:apache-maven:pom:3.1.1) = 3.1.1", sout)

    @mavenprov(["alias/buildroot/usr/share/maven-metadata/alias.xml"])
    def test_alias(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 2)
        assertIn(self, "mvn(jakarta-regexp:jakarta-regexp:pom:) = 1.0", sout)
        assertIn(self, "mvn(regexp:regexp:pom:) = 1.0", sout)

    @mavenprov(["alias2/buildroot/usr/share/maven-metadata/alias2.xml"])
    def test_alias2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 4)
        assertIn(self, "mvn(jakarta-regexp:jakarta-regexp:pom:) = 1.0", sout)
        assertIn(self, "mvn(jakarta-regexp:jakarta-regexp) = 1.0", sout)
        assertIn(self, "mvn(regexp:regexp:pom:) = 1.0", sout)
        assertIn(self, "mvn(regexp:regexp) = 1.0", sout)

    @mavenprov(["compat_alias/buildroot/usr/share/maven-metadata/compat_alias.xml"])
    def test_compat_alias(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 4)
        assertIn(self, "mvn(jakarta-regexp:jakarta-regexp:pom:1.1) = 1.0", sout)
        assertIn(self, "mvn(jakarta-regexp:jakarta-regexp:pom:1.1.1) = 1.0", sout)
        assertIn(self, "mvn(regexp:regexp:pom:1.1) = 1.0", sout)
        assertIn(self, "mvn(regexp:regexp:pom:1.1.1) = 1.0", sout)

    @mavenprov(["jar_extension/buildroot/usr/share/maven-metadata/jar_extension.xml"])
    def test_jar_extension(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 2)
        assertIn(self, "mvn(args4j:args4j:pom:) = 2.0.16", sout)
        assertIn(self, "mvn(args4j:args4j) = 2.0.16", sout)

    @mavenprov(["dashes/buildroot/usr/share/maven-metadata/require.xml"])
    def test_dashes(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 2)
        assertIn(self, "mvn(org.apache.maven:maven-model) = 1.alpha.2", sout)
        assertIn(self, "osgi(osgi1) = 1.5.101.SNAPSHOT", sout)

if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>


def get_mvn_str(gid, aid, ext=None, cla=None, ver=None):
    """Construct and return Maven coordinates.

    At least groupId (gid) and artifactId (aid) needs to be provided.

    >>> get_mvn_str("org.example", "artifact")
    'org.example:artifact'
    >>> get_mvn_str("org.example", "artifact", ver="1")
    'org.example:artifact:1'
    >>> get_mvn_str("org.example", "artifact", cla="test")
    'org.example:artifact::test:'
    """

    mvnstr = "{gid}:{aid}".format(gid=gid, aid=aid)

    # "jar" is a default extension in Maven world, we can omit it
    if ext == "jar":
        ext = ""

    if ext:
        mvnstr = mvnstr + ":{ext}".format(ext=ext)

    if cla:
        if not ext:
            mvnstr = mvnstr + ":"
        mvnstr = mvnstr + ":{cla}".format(cla=cla)

    if ver:
        mvnstr = mvnstr + ":{ver}".format(ver=ver)
    elif cla or ext:
        mvnstr = mvnstr + ":"

    return mvnstr


def get_rpm_str(gid, aid, ext=None, cla=None, namespace=None,
                compat_ver=None, pkg_ver=None):
    """Construct and return string representing RPM's "virtual provide"
    of a given Maven artifact.

    >>> get_rpm_str("org.example", "artifact")
    'mvn(org.example:artifact)'
    >>> get_rpm_str("org.example", "artifact", pkg_ver="1")
    'mvn(org.example:artifact) = 1'
    >>> get_rpm_str("org.example", "artifact", namespace="scl", compat_ver="1")
    'scl-mvn(org.example:artifact:1)'
    """

    mvnstr = get_mvn_str(gid, aid, ext=ext, cla=cla,
                         ver=compat_ver if compat_ver is not None else None)
    rpmstr = "mvn({mvnstr})".format(mvnstr=mvnstr)

    if namespace:
        rpmstr = "{ns}-".format(ns=namespace) + rpmstr

    if pkg_ver:
        pkg_ver = _sanitize_version(pkg_ver)
        rpmstr = rpmstr + " = {ver}".format(ver=pkg_ver)

    return rpmstr


def _sanitize_version(ver):
    """Sanitize version so it can be properly handled by YUM/DNF.

    RPM package managers seem to have a problem with versioned "Provides"
    containing dashes, e.g. "mvn(g:a) = 1.0-SNAPSHOT".

    >>> _sanitize_version("1.0-SNAPSHOT")
    '1.0.SNAPSHOT'
    >>> _sanitize_version("1.0")
    '1.0'
    """
    if ver:
        return ver.replace("-", ".")
    return ""


if __name__ == "__main__":
    import doctest
    doctest.testmod()



#
# Copyright (c) 2014, Red Hat, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Stanislav Ochotnicky <sochotnicky@redhat.com>

from __future__ import print_function

import os
import optparse
import subprocess
import sys

from javapackages.maven.artifact import Artifact
from javapackages.xmvn.xmvn_config import XMvnConfig
from javapackages.common.util import args_to_unicode, command_exists
from javapackages.common.mock import socket_path as mock_socket


def goal_callback(option, opt_str, value, parser):
    assert value is None
    value = []

    for arg in parser.rargs:
        if arg[:2] == "--" and len(arg) > 2:
            break
        if arg[:1] == "-" and len(arg) > 1:
            break
        value.append(arg)

    del parser.rargs[:len(value)]
    setattr(parser.values, option.dest, value)

usage = "usage: %prog [options] [-- maven-arguments]"

if __name__ == "__main__":
    parser = optparse.OptionParser(usage=usage)
    parser.add_option("-b", "--bootstrap", action="store_true",
                      help="Enable bootstrap mode letting XMvn download artifacts online.")
    parser.add_option("-d", "--xmvn-debug", action="store_true",
                      help="Enable debugging output for maven local resolver.")
    parser.add_option("-E", "--disable-effective-poms",
                      action="store_true",
                      help="Disable resolution of effective POMs.")
    parser.add_option("-f", "--force", "--skip-tests",
                      action="store_true",
                      help="Skip test compilation and execution.")
    parser.add_option("-g", "--goal-before", "--pre",
                      dest="goal_before",
                      action="callback",
                      callback=goal_callback,
                      help="Run Maven goals before default XMvn goals.")
    parser.add_option("-G", "--goal-after", "--post",
                      dest="goal_after",
                      action="callback",
                      callback=goal_callback,
                      help="Run Maven goals after default XMvn goals.")
    parser.add_option("--gradle",
                      action="store_true",
                      help="Invoke Gradle instead of Maven.")
    parser.add_option("-i", "--skip-install",
                      action="store_true",
                      help="Skip artifact installation.")
    parser.add_option("-j", "--skip-javadoc",
                      action="store_true",
                      help="Skip javadoc generation and installation.")
    parser.add_option("-s", "--singleton",
                      action="store_true",
                      help="Singleton packaging (one artifact per package).")
    parser.add_option("-X", "--debug",
                      action="store_true",
                      help="Enable Maven debugging output (implies -d).")
    parser.add_option("--xmvn-javadoc",
                      action="store_true",
                      help="Use experimental XMvn javadoc MOJO to generate javadocs.")

    sys.argv = args_to_unicode(sys.argv)

    (options, args) = parser.parse_args()
    xc = XMvnConfig()

    if options.gradle:
        base_goal = "build"
        mvn_args = ["gradle-local", "--no-daemon"]
    else:
        base_goal = "verify"
        mvn_args = ["xmvn", "--batch-mode"]

    if not command_exists(mvn_args[0]):
        if options.gradle:
            print("gradle-local package is not installed, please install it to proceed", file=sys.stderr)
        else:
            # xmvn command is provided by xmvn package, but maven-local
            # pulls in bunch of maven plugins which may come handy
            print("maven-local package is not installed, please install it to proceed", file=sys.stderr)
        sys.exit(1)

    if not options.bootstrap:
        mvn_args.append("--offline")

    if options.disable_effective_poms:
        mvn_args.append("-Dxmvn.compat=20-rpmbuild-raw")

    if options.debug:
        mvn_args.append("--debug")

    if options.xmvn_debug or options.debug:
        mvn_args.append("-Dxmvn.debug")

    if options.force:
        mvn_args.append("-Dmaven.test.skip=true")
        xc.add_custom_option("buildSettings/skipTests", "true")
        if options.gradle:
            base_goal = "assemble"
        else:
            base_goal = "package"

    if mock_socket and os.path.exists(mock_socket):
        interpreter = sys.executable
        java_utils = os.path.dirname(os.path.abspath(__file__))
        cmd = "%s %s/request-artifact.py" % (interpreter, java_utils)
        mvn_args.append("-Dxmvn.resolver.requestArtifactCmd='%s'" % cmd)

    mvn_args.extend(args)

    if options.goal_before:
        mvn_args.extend(options.goal_before)

    mvn_args.append(base_goal)

    if not options.skip_install:
        if options.gradle:
            mvn_args.append("xmvnInstall")
        else:
            mvn_args.append("org.fedoraproject.xmvn:xmvn-mojo:install")

    if not options.skip_javadoc:
        if options.gradle:
            # Automatic javadoc generation for Gradle is not yet implemented in XMvn
            pass
        elif options.xmvn_javadoc:
            mvn_args.append("org.fedoraproject.xmvn:xmvn-mojo:javadoc")
        else:
            mvn_args.append("org.apache.maven.plugins:maven-javadoc-plugin:aggregate")

    if not options.gradle:
        # Build dependency generation for Gradle is not yet implemented in XMvn
        mvn_args.append("org.fedoraproject.xmvn:xmvn-mojo:builddep")

    if options.goal_after:
        mvn_args.extend(options.goal_after)

    if options.singleton:
        # make sure we don't install artifacts with non-empty classifiers
        xc.add_package_mapping(Artifact.from_mvn_str(":::*?:"), "__noinstall",
                               optional=True)
        xc.add_package_mapping(Artifact.from_mvn_str(":{*}"), "@1")

    print("Executing:", " ".join(mvn_args), file=sys.stderr)
    print(mvn_args, file=sys.stderr)
    sys.stderr.flush()
    p = subprocess.Popen(" ".join(mvn_args), shell=True, env=os.environ)
    p.wait()

    subprocess.Popen("""
        if [ -f .xmvn-builddep ]; then
            echo -----BEGIN MAVEN BUILD DEPENDENCIES-----
            gzip -9nc <.xmvn-builddep | base64
            echo -----END MAVEN BUILD DEPENDENCIES-----
        fi
        """, shell=True, env=os.environ).wait()

    sys.exit(p.returncode)



import unittest

from test_common import *


class TestOSGiProv(unittest.TestCase):

    @osgiprov(["/usr/bin/zip"])
    def test_rhbz889131(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 0)

    @osgiprov(["basic/buildroot/usr/share/META-INF/MANIFEST.MF"])
    def test_basic(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        assertIn(self, "osgi(org.junit) = 4.10.0", sout)

    @osgiprov(["basic_jar/buildroot/usr/lib/basic.jar"])
    def test_basic_jar(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        assertIn(self, "osgi(org.junit) = 4.10.0", sout)

    @osgiprov(["empty/buildroot/usr/share/META-INF/MANIFEST.MF"])
    def test_empty(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 0)


if __name__ == '__main__':
    unittest.main()



import unittest

from test_common import *


class TestOSGiReq(unittest.TestCase):

    @osgireq(["basic/buildroot/usr/share/META-INF/MANIFEST.MF"])
    def test_basic(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        assertIn(self, "osgi(org.hamcrest.core)", sout)
        self.assertEqual(len(sout), 1)

    @osgireq(["basic_jar/buildroot/usr/lib/basic.jar"])
    def test_basic_jar(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        assertIn(self, "osgi(org.hamcrest.core)", sout)
        self.assertEqual(len(sout), 1)

    @osgireq(["symlink/buildroot/usr/share/java/foo/META-INF/MANIFEST.MF"])
    def test_symlink(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 0)

    @osgireq(["symlink_jar/buildroot/usr/share/java/foo/basic.jar"])
    def test_symlink_jar(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 0)

    @osgireq(["symlink_dir/buildroot/usr/share/java/foo/META-INF/MANIFEST.MF"])
    def test_symlink_dir(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 0)

    @osgireq(["symlink_dir_jar/buildroot/usr/share/java/foo/basic.jar"])
    def test_symlink_dir_jar(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 0)

    @osgireq(["empty/META-INF/MANIFEST.MF"])
    def test_empty(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 0)

    @osgireq(["self_require/buildroot/usr/share/java/foo/META-INF/MANIFEST.MF"])
    def test_self_require(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        assertIn(self, "osgi(bar) = 4.5.6", sout)
        self.assertEqual(len(sout), 1, sout)

    @osgireq(["corrupt/buildroot/usr/share/java/corrupt.jar"])
    def test_corrupt(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        self.assertEqual(len(sout), 0, sout)

if __name__ == '__main__':
    unittest.main()



import unittest
import os
import io

from shutil import copytree
from shutil import rmtree

from test_common import assertIn
from test_rpmbuild import Package
from xml_compare import compare_xml_files

DIRPATH = os.path.dirname(os.path.realpath(__file__))

DATADIR = os.path.join(DIRPATH, "data", "pom_macros")
WORKDIR = os.path.join(DATADIR, "..", 'pom_macros_workdir')


def exec_macro(command = "", pom = "pom.xml", tail=''):
    def test_decorator(function):
        def test_decorated(self, *args, **kwargs):
            pompath = os.path.join(WORKDIR, pom)
            pomname = os.path.basename(pom)
            package = Package(function.__name__)
            package.add_source(pompath)
            pomsourcepath = os.path.join(package.buildpath, pomname)
            package.append_to_prep('%{command} {pom} {tail}'.format(command=command,
                pom=pomname, tail=tail))
            stdin, stderr, return_value = package.run_prep()

            function(self, stdin, stderr, return_value, pomsourcepath)
        return test_decorated
    return test_decorator

def check_result(pom_path):
    wantpath = '{pom}-want'.format(pom=os.path.basename(pom_path))
    report = compare_xml_files(pom_path, wantpath)
    res = not report
    return report, res

def get_result_literally(pom_path, encoding='UTF-8'):
    with io.open(pom_path, 'r', encoding=encoding) as gotfile:
        got = gotfile.read().split('\n')

    wantpath = '{pom}-want'.format(pom=os.path.basename(pom_path))
    with io.open(wantpath, 'r', encoding=encoding) as wantfile:
        want = wantfile.read().split('\n')
    return got, want


class PomMacrosTest(unittest.TestCase):
    olddir = os.getcwd()
    maxDiff = 2048

    @classmethod
    def setUpClass(cls):
        cls.tearDownClass()
        cls.olddir = os.getcwd()
        copytree(DATADIR, WORKDIR)
        os.chdir(WORKDIR)

    @classmethod
    def tearDownClass(cls):
        try:
            rmtree(WORKDIR)
        except OSError:
            pass
        os.chdir(cls.olddir)

    def test_usage(self):
        package = Package("usage")
        pompath = os.path.join(WORKDIR, "pom_remove_dep.xml")
        package.add_source(pompath, "pom.xml")
        package.append_to_prep('%pom_remove_dep')
        _, stderr, return_value = package.run_prep()
        self.assertEqual(return_value, 1)
        assertIn(self, "Usage: %pom_remove_dep "\
                      "[groupId]:[artifactId] [POM location]", stderr)

    @exec_macro("pom_remove_dep :commons-io", "pom_remove_dep.xml")
    def test_remove_dep(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_remove_dep commons-io:commons-io", "pom_remove_dep2.xml")
    def test_remove_deps(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_remove_dep :", "pom_remove_dep_wildcard.xml")
    def test_remove_wildcard(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_remove_dep junit:", "pom_remove_dep_nons.xml")
    def test_remove_dep_nons(self, stdin, stderr, returncode, pom_path):
        # no namespace
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_remove_dep not:there", "pom_remove_dep_ret1.xml")
    def test_remove_dep_no_effect(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 1, stderr)
        assertIn(self, "Error in processing", stderr)

    @exec_macro("pom_remove_plugin :my-plugin", "pom_remove_plugin.xml")
    def test_remove_plugin(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_remove_plugin my.group:my-plugin",
                "pom_remove_plugin2.xml")
    def test_remove_plugins(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_remove_plugin :my-plugin", "pom_remove_plugin_nons.xml")
    def test_remove_plugin_nons(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_remove_plugin not:there", "pom_remove_plugin_ret1.xml")
    def test_remove_plugin_ret1(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 1)
        assertIn(self, "Error in processing", stderr)

    @exec_macro("pom_disable_module module", "pom_disable_module.xml")
    def test_disable_module(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_disable_module module", "pom_disable_module_nons.xml")
    def test_disable_module_nons(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_disable_module notthere", "pom_disable_module_ret1.xml")
    def test_disable_module_no_effect(self, stdin, stderr,
                                      returncode, pom_path):
        self.assertEqual(returncode, 1)
        assertIn(self, "Error in processing", stderr)

    @exec_macro("pom_add_dep gdep:adep:3.2:test", "pom_add_dep.xml")
    def test_add_dep(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_dep foo:bar:1.2", "pom_add_dep_extra.xml",
                "'<scope>test</scope>'")
    def test_add_dep_extra(self, stdout, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_dep_mgmt foo:bar:1.2", "pom_add_dep_mgmt_extra.xml",
                "'<scope>test</scope>'")
    def test_add_dep_mgmt_extra(self, stdout, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_dep gdep:adep:3.2:test", "pom_add_dep_whitespace.xml")
    def test_add_dep_whitespace(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        got, want = get_result_literally(pom_path)
        self.assertEqual(got, want)

    @exec_macro("pom_add_dep gdep:adep", "pom_add_dep2.xml")
    def test_add_dep2(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_dep A:B::compile", "pom_add_dep_nons.xml")
    def test_add_dep_nons(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_dep_mgmt gdep:adep:3.2:test", "pom_add_dep_mgmt.xml")
    def test_add_dep_mgmt(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_dep_mgmt gdep:adep", "pom_add_dep_mgmt2.xml")
    def test_add_dep_mgmt2(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_dep_mgmt gdep:adep", "pom_add_dep_mgmt_nons.xml")
    def test_add_dep_mgmt_nons(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_plugin plugin:plug:3", "pom_add_plugin.xml")
    def test_add_plugin(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_plugin plugin:plug:3", "pom_add_plugin_extra.xml",
                """'<executions>
                       <id>default-compile</id>
                       <goals>
                          <goal>testCompile</goal>
                       </goals>
                    </executions>
                   '""")
    def test_add_plugin_extra(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_plugin plug:plug", "pom_add_plugin2.xml")
    def test_add_plugin2(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_plugin g:a:15", "pom_add_plugin_nons.xml")
    def test_add_plugin_nons(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_plugin :A-A", "pom_add_plugin_default_gid.xml")
    def test_add_plugin_default_gid(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_remove_parent", "pom_remove_parent.xml")
    def test_remove_parent(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_remove_parent", "pom_remove_parent_nons.xml")
    def test_remove_parent_nons(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_remove_parent", "pom_remove_parent_fail.xml")
    def test_remove_parent_fail(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 1)
        assertIn(self, "Error in processing", stderr)

    @exec_macro("pom_add_parent pg:pa:21", "pom_add_parent.xml")
    def test_add_parent(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_add_parent pg:pa", "pom_add_parent_nons.xml")
    def test_add_parent_nons(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_set_parent pg:aa:5", "pom_set_parent.xml")
    def test_set_parent(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_set_parent pg:aa", "pom_set_parent_nons.xml")
    def test_set_parent_nons(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_set_parent pg:aa", "pom_set_parent_fail.xml")
    def test_set_parent_fail(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 1, stderr)
        assertIn(self, "Error in processing", stderr)

    @exec_macro("pom_xpath_remove pom:maven-old", "pom_xpath_remove.xml")
    def test_xpath_remove(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_remove 'pom:modules/node()'", "pom_xpath_remove_generic.xml")
    def test_xpath_remove_generic(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_remove \
                 \"pom:dependency[pom:artifactId[text()='m-m']]\"",
                "pom_xpath_remove2.xml")
    def test_xpath_remove2(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_remove pom:dependencies",
                "pom_xpath_remove_nons.xml")
    def test_xpath_remove_nons(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_remove pom:not/pom:there",
                "pom_xpath_remove_fail.xml")
    def test_xpath_remove_fail(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 1, stderr)
        assertIn(self, "Error in processing", stderr)

    @exec_macro("pom_xpath_inject pom:parent '<version>1.2</version>'",
                "pom_xpath_inject.xml")
    def test_xpath_inject(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_inject pom:dependencies/pom:dependency "
                "'<classifier>test</classifier>'",
                "pom_xpath_inject_multiple.xml")
    def test_xpath_inject_multiple(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_inject pom:parent '<version>1.2</version>'",
                "pom_xpath_inject_whitespace.xml")
    def test_xpath_inject_whitespace(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        got, want = get_result_literally(pom_path)
        self.assertEqual(got, want)

    @exec_macro("pom_xpath_inject pom:parent '<version>1.2</version>'",
                "pom_xpath_inject_whitespace1.xml")
    def test_xpath_inject_whitespace1(self, stdin, stderr,
                                      returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        got, want = get_result_literally(pom_path)
        self.assertEqual(got, want)

    @exec_macro("pom_xpath_inject pom:build/pom:plugins '<plugin>\
                 <groupId>some</groupId>\
                 <artifactId>plugin</artifactId></plugin>'",
                "pom_xpath_inject2.xml")
    def test_xpath_inject2(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_inject pom:project '<name>Commons Lang</name>'",
                "pom_xpath_inject_nons.xml")
    def test_xpath_inject_nons(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_inject pom:project '<name>Commons Lang<name>'",
            "pom_xpath_inject_fail.xml")
    def test_xpath_inject_fail(self, stdin, stderr, returncode, pom_path):
        # invalid XML code
        self.assertEqual(returncode, 1, stderr)
        assertIn(self, "Error in processing", stderr)

    @exec_macro("pom_xpath_replace pom:parent/pom:groupId \
                 '<groupId>commons</groupId>'",
                "pom_xpath_replace.xml")
    def test_xpath_replace(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_replace 'pom:parent/pom:groupId/text()' 'commons'",
                "pom_xpath_replace_text.xml")
    def test_xpath_replace_text(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        got, want = get_result_literally(pom_path)
        self.assertEqual(got, want)

    @exec_macro("pom_xpath_replace //pom:dependency/pom:groupId "
                "'<groupId>a</groupId>'",
                "pom_xpath_replace_multiple.xml")
    def test_xpath_replace_multiple(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_replace //pom:dependency/pom:groupId "
                "'<groupId>a/groupId>'",
                "pom_xpath_replace_multiple.xml")
    def test_xpath_replace_invalid(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 1)
        assertIn(self, "Error in processing", stderr)

    @exec_macro("pom_xpath_replace pom:parent/pom:groupId \
                 '<groupId>commons</groupId>'",
                "pom_xpath_replace_whitespace.xml")
    def test_xpath_replace_whitespace(self, stdin, stderr,
                                      returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        got, want = get_result_literally(pom_path)
        self.assertEqual(got, want)

    @exec_macro("pom_xpath_replace \
                 \"pom:dependency[pom:artifactId[text()='junit']]/pom:version\"\
                 '<version>commons</version>'", "pom_xpath_replace2.xml")
    def test_xpath_replace2(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_replace pom:parent \
                 '<groupId>org.apache.commons</groupId>'",
                "pom_xpath_replace_nons.xml")
    def test_xpath_replace_nons(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_replace pom:project/pom:not/pom:there \
                 '<groupId>commons</groupId>'",
                "pom_xpath_replace_fail.xml")
    def test_xpath_replace_fail(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 1, stderr)
        assertIn(self, "Error in processing", stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_replace pom:project/pom:groupId \
                 '<groupId>commons<groupId>'",
                "pom_xpath_replace_invalid.xml")
    def test_xpath_replace_invalid_xml(self, stdin, stderr,
                                       returncode, pom_path):
        self.assertEqual(returncode, 1, stderr)
        assertIn(self, "Error in processing", stderr)

    @exec_macro("pom_xpath_set pom:project/pom:groupId 'commons'",
            "pom_xpath_set.xml")
    def test_xpath_set(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_set pom:groupId 'commons'",
            "pom_xpath_set_multiple.xml")
    def test_xpath_set_multiple(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_set \"pom:build/pom:plugins/pom:plugin[pom:groupId\
                 [text()='org.codehaus.mojo']]/pom:version\" 2",
                "pom_xpath_set2.xml")
    def test_xpath_set2(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_set pom:parent/pom:artifactId 'a-Id'",
                "pom_xpath_set_nons.xml")
    def test_xpath_set_nons(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_set pom:project/pom:parent/pom:notThere Project",
                "pom_xpath_set_fail.xml")
    def test_xpath_set_fail(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 1, stderr)
        assertIn(self, "Error in processing", stderr)

        report, res = check_result(pom_path)
        self.assertEqual(res, True, report)

    @exec_macro("pom_xpath_set pom:project/pom:version 2.7", "pom_keep_xml_declaration.xml")
    def test_keep_xml_declaration(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        got, want = get_result_literally(pom_path)
        self.assertEqual(got, want)

    @exec_macro("pom_xpath_set pom:project/pom:version 2.7", "pom_keep_no_xml_declaration.xml")
    def test_keep_no_xml_declaration(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        got, want = get_result_literally(pom_path)
        self.assertEqual(got, want)

    @exec_macro("pom_xpath_set pom:project/pom:version 2.7", "pom_unicode.xml")
    def test_unicode(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        got, want = get_result_literally(pom_path)
        self.assertEqual(got, want)

    @exec_macro("pom_xpath_set pom:project/pom:version 2.7", "pom_non_unicode.xml")
    def test_non_unicode(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        got, want = get_result_literally(pom_path, encoding='ISO-8859-1')
        self.assertEqual(got, want)

    @exec_macro("pom_remove_parent", "unparsable_xml.pom")
    def test_unparsable_xml(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 1, stderr)
        assertIn(self, "Error in processing", stderr)

    def test_no_pom(self):
        package = Package("nonexistent_pom")
        package.append_to_prep('%pom_remove_parent')
        _, stderr, returncode = package.run_prep()

        self.assertEqual(returncode, 1, stderr)
        assertIn(self, "Couldn't locate ", stderr)

    def test_no_pom_explicit(self):
        package = Package("nonexistent_pom_explicit")
        package.append_to_prep('%pom_remove_parent nonexistent_pom.xml')
        _, stderr, returncode = package.run_prep()

        self.assertEqual(returncode, 1, stderr)
        assertIn(self, "Couldn't locate ", stderr)

    @exec_macro("pom_change_dep : commons-nio:commons-nio", "pom_change_dep_nover.xml")
    def test_change_dep_nover(self, stdin, stderr, returncode, pom_path):
        self.assertEqual(returncode, 0, stderr)

        got, want = get_result_literally(pom_path)
        self.assertEqual(got, want)


if __name__ == '__main__':
    unittest.main()



import unittest
import shutil
import os
from test_common import javautils_script, get_config_file_list, \
        get_actual_config, get_expected_config, DIRPATH

from xml_compare import compare_xml_files

class TestMvnFile(unittest.TestCase):

    maxDiff = 2048

    def setUp(self):
        self.olddir = os.getcwd()
        self.workdir = os.path.join(DIRPATH, 'workdir')
        os.mkdir(self.workdir)
        os.chdir(self.workdir)

    def tearDown(self):
        try:
            shutil.rmtree(self.workdir)
        except OSError:
            pass
        os.chdir(self.olddir)

    @javautils_script('mvn_file', [])
    def test_run_no_args(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertEqual("Usage:", stderr[:6])

    @javautils_script('mvn_file', ['-h'])
    def test_help(self, stdout, stderr, return_value):
        self.assertTrue(stdout)

    @javautils_script('mvn_file',['x', ])
    def test_single(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_file',[':x', 'file', ])
    def test_simple(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'simple'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',[':guice', 'google/guice', 'guice', ])
    def test_symlink(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'symlink'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',['a:b', 'file', ])
    def test_group(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'group'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',[':a:1.3', 'file', ])
    def test_version(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'version'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',['a:b:c:', 'file', ])
    def test_extension(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'extension'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',['*:a', 'file', ])
    def test_wildcard(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'wildcard'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',['a', 'file', ])
    def test_invalid1(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_file',['a:b:c:d:e:f', 'file', ])
    def test_invalid2(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_file',[':a', 'file', 'sym1', 'sym2', 'sym3', ])
    def test_symlinks(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'symlinks'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',['a:b:c:d:1', 'sym', ])
    def test_classifier(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'classifier'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',['a::c:', 'sym', ])
    def test_wildcard2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'wildcard2'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',['a:b', 'sym1', 'sym2', 'sym3', 'sym4', 'sym5',
        'sym6', 'sym7', 'sym8', 'sym9', 'sym10', 'sym11', 'sym12', 'sym13',
        'sym14', 'sym15', 'sym16', 'sym17', 'sym18', 'sym19', 'sym20', 'sym21'])
    def test_more_symlinks(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'more_symlinks'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',[':{aa,bb}', '@1', ])
    def test_backref(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'backref'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',['{aa,bb}:{cc,dd}', '@2', '@1', ])
    def test_backref1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'backref1'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',[':a', '@1', ])
    def test_backref2(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_file',[':{a,b}', '@1', '@4', ])
    def test_backref3(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_file',['{aa,bb}:{x,y', '@1', ])
    def test_odd_braces1(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_file',['{aa,bb}:{x,y}}', '@1', ])
    def test_odd_braces2(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_file',['a:b', 'a/file1', ])
    def test_relative1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'relative1'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',['a:b', '../file1', ])
    def test_relative2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'relative2'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',['a:{bb,cc}', 'a/@1', ])
    def test_relative3(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'relative3'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',['a:b', 'file', '/usr/share/java/sym'])
    def test_absolute1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_file', 'absolute1'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_file',['a:b', '/usr/share/java/sym', ])
    def test_absolute2(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

if __name__ == '__main__':
    unittest.main()



import unittest
import shutil
import os
from test_common import javautils_script, get_config_file_list, \
        get_actual_config, get_expected_config, DIRPATH

from xml_compare import compare_xml_files

class TestMvnPackage(unittest.TestCase):

    maxDiff = 2048

    def setUp(self):
        self.olddir = os.getcwd()
        self.workdir = os.path.join(DIRPATH, 'workdir')
        shutil.rmtree(self.workdir, ignore_errors=True)
        os.mkdir(self.workdir)
        os.chdir(self.workdir)

    def tearDown(self):
        try:
            shutil.rmtree(self.workdir)
        except OSError:
            pass
        os.chdir(self.olddir)

    @javautils_script('mvn_package', [])
    def test_run_no_args(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertEqual("Usage:", stderr[:6])

    @javautils_script('mvn_package', ['-h'])
    def test_help(self, stdout, stderr, return_value):
        self.assertTrue(stdout)

    @javautils_script('mvn_package',['aaa:bbb', 'pack', ])
    def test_simple(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'simple'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',['aaa:bbb:1.2', 'pack', ])
    def test_version(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'version'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',['aaa:bbb:ccc:', 'pack', ])
    def test_extension(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'extension'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',['aaa:bbb:ccc:ddd:', 'pack', ])
    def test_classifier(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'classifier'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',['aaa:bbb:ccc:ddd:21', 'pack', ])
    def test_all(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'all'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',[':bbb', 'pack', ])
    def test_wildcard1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'wildcard1'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',[':', 'pack', ])
    def test_wildcard2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'wildcard2'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',['*:*', 'pack', ])
    def test_wildcard3(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'wildcard3'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',[':bbb-{a,b,c}', 'pack', ])
    def test_braces1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'braces1'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',['a-{b,c}:{x,y}-z', 'pack', ])
    def test_braces2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'braces2'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',['aaa:bbb', ])
    def test_single(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'single'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',['aaa:bbb', 'pack', 'evil', ])
    def test_more(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_package',[':bbb-{a,b', 'pack', ])
    def test_odd_braces(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_package',[':bb-a,b}', 'pack', ])
    def test_odd_braces1(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_package',['a-{b,c}:{x,y}-z', '@2', ])
    def test_backref1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'backref1'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',['a-{b,c}:{x,y}-z', '@1', ])
    def test_backref2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'backref2'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_package',['a:b', '@1', ])
    def test_backref3(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_package',['a-{b,c}:x', '@3', ])
    def test_backref4(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_package',['aaa:bbb', '__noinstall', ])
    def test_noinstall(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_package', 'noinstall'),
                 ['artifactGlob'])
            self.assertFalse(report, '\n' + report)

if __name__ == '__main__':
    unittest.main()
